import { GraphQueryResult } from '../../data-access/store/graphQueryResultSlice';
import { ML } from '../../data-access/store/mlSlice';
import { SchemaGraph } from '../../schema';
import type { AppDispatch } from '../../data-access';
import { FC } from 'react';
import { Visualizations } from '../hooks';
import { Edge, Node } from 'reactflow';

export type VisualizationConfiguration = { [id: string]: any };

export type VISComponentType = {
  displayName: keyof typeof Visualizations;
  component: FC<any>;
  settings: FC<any>;
  configuration: { [id: string]: any };
};

export type VisualizationPropTypes = {
  data: GraphQueryResult;
  schema: SchemaGraph;
  ml: ML;
  configuration: VisualizationConfiguration;
  dispatch: AppDispatch;
  handleHover: (val: any) => void;
  handleSelect: (val: any) => void;
};

export type SchemaElements = {
  nodes: Node[];
  edges: Edge[];
  selfEdges: Edge[];
};

export interface Point {
  x: number;
  y: number;
}

export interface BoundingBox {
  topLeft: Point;
  bottomRight: Point;
}

export interface NodeQualityDataForEntities {
  nodeCount: number;
  attributeNullCount: number;
  notConnectedNodeCount: number;
  isAttributeDataIn: boolean; // is true when the data to display has arrived
  onClickCloseButton: () => void;
}

export interface NodeQualityDataForRelations {
  nodeCount: number;
  attributeNullCount: number;
  fromRatio: number; // the ratio of from-entity nodes to nodes that have this relation
  toRatio: number; // the ratio of to-entity nodes to nodes that have this relation
  isAttributeDataIn: boolean; // is true when the data to display has arrived
  onClickCloseButton: () => void;
}

export interface NodeQualityPopupNode extends Node {
  data: NodeQualityDataForEntities | NodeQualityDataForRelations;
  nodeID: string; //ID of the node for which the popup is
}

export interface AttributeAnalyticsData {
  nodeType: NodeType;
  nodeID: string;
  attributes: AttributeWithData[];
  isAttributeDataIn: boolean; // is true when the data to display has arrived
  onClickCloseButton: () => void;
  onClickPlaceInQueryBuilderButton: (name: string, type: string) => void;
  searchForAttributes: (id: string, searchbarValue: string) => void;
  resetAttributeFilters: (id: string) => void;
  applyAttributeFilters: (id: string, category: AttributeCategory, predicate: string, percentage: number) => void;
}

export enum AttributeCategory {
  categorical = 'Categorical',
  numerical = 'Numerical',
  other = 'Other',
  undefined = 'undefined',
}

export enum NodeType {
  entity = 'entity',
  relation = 'relation',
}

export interface AttributeAnalyticsPopupMenuNode extends Node {
  nodeID: string; //ID of the node for which the popup is
  data: AttributeAnalyticsData;
}

export type AttributeWithData = {
  attribute: any;
  category: AttributeCategory;
  nullAmount: number;
};
