import React from 'react';
import { layerTypes } from './layers';
import { Layer } from '../mapvis.types';
import { makeLayer } from '../utlis';

type Props = {
  layers: Layer[];
  setLayers: React.Dispatch<React.SetStateAction<Layer[]>>;
  graphInfo: Record<string, any>;
  setShowFilter: React.Dispatch<React.SetStateAction<boolean>>;
};

export function LayerPanel({ layers, setLayers, graphInfo, setShowFilter }: Props) {
  const [newLayerType, setNewLayerType] = React.useState<string>('');

  const createLayer = (type: string) => {
    setNewLayerType('');
    const newLayer = makeLayer(type);
    setLayers([...layers, newLayer]);
  };

  const updateLayer = (id: number, newParams: Record<string, any>) => {
    setLayers(layers.map((layer) => (layer.id === id ? { ...layer, ...newParams } : layer)));
  };

  const deleteLayer = (id: number) => {
    setLayers(layers.filter((layer) => layer.id !== id));
  };

  return (
    <div className="w-3/10 bg-light flex flex-col h-full">
      <div className="p-3 flex justify-center font-bold">
        <h4>Layer Configurations</h4>
      </div>

      <div className="divider m-0"></div>

      <div className="p-3">
        <select className="select select-bordered w-full" value={newLayerType} onChange={(e) => setNewLayerType(e.target.value)}>
          <option disabled selected>
            Layer type
          </option>
          {Object.keys(layerTypes).map((type) => (
            <option key={type} value={type}>
              {type}
            </option>
          ))}
        </select>

        <button className="btn btn-primary w-full mt-1" onClick={() => createLayer(newLayerType)} disabled={!newLayerType}>
          Add Layer
        </button>
      </div>

      <div className="divider m-0"></div>

      <div className="p-2 grow flex flex-col overflow-hidden">
        <p>Displayed Layers</p>
        <div className="grow overflow-y-auto">
          {layers.length == 0 ? (
            <p>No layers yet, add a layer above...</p>
          ) : (
            <>
              {layers.map((layer) => (
                <div className="collapse  bg-secondary-200">
                  <input type="radio" name="my-accordion-2" />
                  <div className="collapse-title text-xl font-medium">{`${layer.type.type} layer`}</div>
                  <div className="collapse-content">{layer.type.generateLayerOptions(layer, updateLayer, graphInfo, deleteLayer)}</div>
                </div>
              ))}
            </>
          )}
        </div>
      </div>

      <div className="divider m-0"></div>

      <div className="p-3">
        <button className="btn btn-primary w-full" onClick={() => setShowFilter(true)}>
          Filter
        </button>
        <button className="btn btn-secondary w-full my-1">Export layer settings</button>
      </div>
    </div>
  );
}
